// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AssetManager.hpp"
#include "loaders/TextureLoader.hpp"

using namespace Dawn;

void AssetManager::init() {

}

void AssetManager::update() {
  auto itPending = pendingAssetLoaders.begin();
  while(itPending != pendingAssetLoaders.end()) {
    auto loader = *itPending;
    loader->updateSync();
    loader->updateAsync();
    if(loader->loaded) {
      finishedAssetLoaders.push_back(loader);
      itPending = pendingAssetLoaders.erase(itPending);
    } else {
      itPending++;
    }
  }
}

void AssetManager::removeExisting(const std::string filename) {
  auto existing = std::find_if(
    pendingAssetLoaders.begin(), pendingAssetLoaders.end(),
    [&](auto &loader) {
      return loader->name == filename;
    }
  );
  if(existing != pendingAssetLoaders.end()) {
    pendingAssetLoaders.erase(existing);
  }

  existing = std::find_if(
    finishedAssetLoaders.begin(), finishedAssetLoaders.end(),
    [&](auto &loader) {
      return loader->name == filename;
    }
  );
  if(existing != finishedAssetLoaders.end()) {
    finishedAssetLoaders.erase(existing);
  }
}

template<>
std::shared_ptr<Texture> AssetManager::get<Texture>(
  const std::string filename
) {
  
  auto existing = this->getExisting<TextureLoader>(filename);
  if(existing) {
    std::cout << "Found existing texture loader for " << filename << std::endl;

    // Check pointer hasn't gone stale, if it has remove it and create new.
    auto texture = existing->weakTexture.lock();
    if(texture) return texture;

    std::cout << "Texture loader for " << filename << " has gone stale." << std::endl;
    this->removeExisting(filename);
  }

  std::shared_ptr<TextureLoader> loader = std::make_shared<TextureLoader>(
    filename
  );
  pendingAssetLoaders.push_back(std::static_pointer_cast<AssetLoader>(loader));
  return loader->sharedTexture;
}

AssetManager::~AssetManager() {
  
}