// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/Texture.hpp"
#include "TrueTypeCharacter.hpp"
#include <ft2build.h>
#include FT_FREETYPE_H

namespace Dawn {
  class TrueTypeTexture final {
    public:
      FT_Face face;
      std::shared_ptr<Texture> texture;
      uint32_t fontSize;
      uint8_t style;
      std::unordered_map<wchar_t, struct TrueTypeCharacter> characterData;

      /**
       * Construct a new New True Type Face Texture object
       * 
       * @param face The freetype face object.
       * @param style Style that this font has, used for locking.
       */
      TrueTypeTexture(
        FT_Face face,
        uint32_t fontSize,
        uint8_t style
      );

      /**
       * Returns the character data for the given character.
       * 
       * @param c Character to get data for.
       * @return The Character data for the given character.
       */
      struct TrueTypeCharacter getCharacterData(wchar_t c);

      /**
       * Destroys this true type face texture.
       */
      ~TrueTypeTexture();
  };
}