// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "QuadMesh.hpp"

using namespace Dawn;

void QuadMesh::buffer(
  const std::shared_ptr<Mesh> mesh,
  const glm::vec4 positions,
  const glm::vec4 coordinates,
  const int32_t verticeStart,
  const int32_t indiceStart
) {
  glm::vec3 vertices[QUAD_VERTICE_COUNT] = {
    glm::vec3(positions.x, positions.y, 0),
    glm::vec3(positions.z, positions.y, 0),
    glm::vec3(positions.x, positions.w, 0),
    glm::vec3(positions.z, positions.w, 0)
  };

  glm::vec2 coords[QUAD_VERTICE_COUNT] = {
    glm::vec2(coordinates.x, coordinates.y),
    glm::vec2(coordinates.z, coordinates.y),
    glm::vec2(coordinates.x, coordinates.w),
    glm::vec2(coordinates.z, coordinates.w)
  };

  int32_t indices[QUAD_INDICE_COUNT] = {
    verticeStart, verticeStart + 1, verticeStart + 3,
    verticeStart, verticeStart + 2, verticeStart + 3
  };

  mesh->bufferPositions(verticeStart, vertices, QUAD_VERTICE_COUNT);
  mesh->bufferCoordinates(verticeStart, coords, QUAD_VERTICE_COUNT);
  mesh->bufferIndices(indiceStart, indices, QUAD_INDICE_COUNT);
}