// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scene/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"
#include "display/font/TrueTypeTexture.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "display/mesh/QuadMesh.hpp"

#include <ft2build.h>
#include FT_FREETYPE_H

using namespace Dawn;

FT_Library fontLibrary;
FT_Face face;
std::shared_ptr<TrueTypeTexture> texture;

void Dawn::helloWorldScene(Scene &s) {
  int32_t ret = FT_Init_FreeType(&fontLibrary);
  assertTrue(ret == 0, "Failed to initialize FreeType library");
  ret = FT_New_Face(fontLibrary,
    // "/usr/share/fonts/TTF/arial.ttf",
    "/home/yourwishes/Downloads/Noto_Sans_JP/static/NotoSansJP-Regular.ttf",
    0,
    &face
  );
  assertTrue(ret == 0, "Failed to load font face");
  texture = std::make_shared<TrueTypeTexture>(face, 128, 0);

  auto test = texture->getCharacterData(L'あ');

  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  // cameraItem->lookAt({ 5, 5, 5 }, { 0, 0, 0 }, { 0, 1, 0 });
  cameraItem->lookAt({ 0, 0, 3 }, { 0, 0, 0 }, { 0, 1, 0 });

  auto quad = s.createSceneItem();
  auto quadMesh = std::make_shared<Mesh>();
  quadMesh->createBuffers(QUAD_VERTICE_COUNT, QUAD_INDICE_COUNT);
  QuadMesh::buffer(
    quadMesh,
    glm::vec4(0, 0, test.size.x, test.size.y) / 128.0f,
    test.quad,
    0,
    0
  );
  auto quadRenderer = quad->addComponent<MeshRenderer>();
  quadRenderer->mesh = quadMesh;

  auto quadMaterial = quad->addComponent<SimpleTexturedMaterial>();
  quadMaterial->setTexture(texture->texture);
}